/*
 * Copyright 2010, cluster:Systems CSG GmbH
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.sourceforge.mailprobe.smtp;

import java.util.Hashtable;
import java.util.TreeMap;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

/**
 * Class for lookup of MX-Records for a given domain.
 * Code was inspired by 'Lookup using MX record to validate mail server',
 * http://www.rgagnon.com/javadetails/java-0452.html
 *
 * @author Thomas Tesche <thomas.tesche@clustersystems.de>
 */
public class MXLookup {

  public TreeMap<Integer, String> getMXHosts(String mailDomainName) throws NamingException {

    // Perform a DNS lookup for MX records
    Hashtable env = new Hashtable();
    env.put("java.naming.factory.initial",
            "com.sun.jndi.dns.DnsContextFactory");
    DirContext dirContext = new InitialDirContext(env);

    Attributes attrs = dirContext.getAttributes(mailDomainName, new String[]{"MX"});
    Attribute attr = attrs.get("MX");

    // if we don't have an MX record, try the machine itself
    if ((attr == null) || (attr.size() == 0)) {
      attrs = dirContext.getAttributes(mailDomainName, new String[]{"A"});
      attr = attrs.get("A");
      if (attr == null) {
        throw new NamingException("No finding for name '" + mailDomainName + "'");
      }
    }

    TreeMap<Integer, String> result = new TreeMap<Integer, String>();

    NamingEnumeration namingEnumeration = attr.getAll();

    while (namingEnumeration.hasMore()) {
      String mailhost;
      Integer priority = new Integer(0);

      String x = (String) namingEnumeration.next();
      String f[] = x.split(" ");

      if (f.length == 1) {
        mailhost = f[0];
        priority = new Integer(0);

      } else if (f[1].endsWith(".")) {
        mailhost = f[1].substring(0, (f[1].length() - 1));
        priority = new Integer(f[0]);

      } else {
        mailhost = f[1];
        priority = new Integer(0);

      }

      result.put(priority, mailhost);

    }

    if (result.size() == 0) {
      // The mailDoaminName doesn't exist in the DNS
      throw new NamingException("No finding for name '" + mailDomainName + "'");
    }

    return result;
  }
}
