/*
 * Copyright 2010, cluster:Systems CSG GmbH
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package net.sourceforge.mailprobe.util;

import java.util.ArrayList;
import java.util.List;

/**
 * Enum for all mail protocolls.
 * 
 * @author Thomas Tesche <thomas.tesche@clustersystems.de>
 */
public enum Protocol {

  SMTP("smtp"), SMTPS("smtps"), IMAP("imap"), IMAPS("imaps"), POP3("pop3"), POP3S("pop3s");
  private String proto;
  private static List nameList;
  private static List protoList;

  Protocol(String name) {
    this.proto = name;
  }

  @Override
  public String toString() {
    return proto;
  }

  public String value() {
    return name();
  }

  public static List namelist() {
    if (nameList != null) {
      return nameList;
    } else {
      buildList();
      return nameList;
    }
  }

  public static List list() {
    if (protoList != null) {
      return protoList;
    } else {
      buildList();
      return protoList;
    }
  }

  private static synchronized List buildList() {
    // List was not initialized...
    nameList = new ArrayList();
    protoList = new ArrayList();
    for (Protocol c : Protocol.values()) {
      nameList.add(c.name());
      protoList.add(c.proto);
    }
    return nameList;
  }

  public static Protocol fromValue(String v) {
    return valueOf(v);
  }
}
